import os from "os";
import fs from "fs";
import path from "path";
import { C } from "./common";



const URLS = {
	GITHUB: `https://github.com/kangkang520/node-addons/releases/download/cv{{version}}/{{filename}}`,
	URNAS: `http://git.urnas.cn:5200/yizhi-js-lib/opencv/releases/download/{{version}}/{{filename}}`,
}

function releaseVersion() { return require("../../package.json").releaseVersion }

function getURL(template: string) {
	const URL_CONFIG: Record<string, Record<string, string>> = {
		"win32": {
			"x64": "cv_windows_x64.node",
		},
		"linux": {
			"x64": "cv_linux_x64.node",
			"arm64": "cv_linux_arm64.node",
		},
		"darwin": {
			"arm64": "cv_macos_arm64.node"
		}
	};

	const archConfig = URL_CONFIG[os.platform()];
	if (!archConfig) throw new Error(`Unsupported platform: ${os.platform()}`);
	const downloadName = archConfig[os.arch()];
	if (!downloadName) throw new Error(`Unsupported architecture: ${os.arch()}`);
	return template.replaceAll("{{version}}", releaseVersion()).replaceAll("{{filename}}", downloadName);
}

async function getStream() {
	for (const [name, url] of Object.entries(URLS)) {
		try {
			return await fetch(getURL(url)).then(res => {
				if (res.status != 200) throw new Error("Failed to download addon.");
				return res.blob().then(b => b.stream());
			})
		} catch (e) { }
	}
	throw new Error("Failed to download addon.");
}

export async function downloadAddon(savename?: string) {
	const defaultAddon = path.resolve(process.cwd(), C("ADDON_PATH"));
	const saveName = savename ? path.resolve(path.dirname(defaultAddon), savename) : defaultAddon;
	if (fs.existsSync(saveName)) {
		try {
			const addon = require(saveName);
			if (addon.__release__ === releaseVersion()) return saveName;
			//清除缓存
			delete require.cache[saveName];
		} catch (err) { }
	}

	await fs.promises.mkdir(path.dirname(saveName), { recursive: true });

	const stream = await getStream();
	const cacheFile = await new Promise<string>((resolve, reject) => {
		const cacheFile = path.join(os.tmpdir(), Date.now() + ".cv.node");
		let fsStream!: ReturnType<typeof fs.createWriteStream>;
		stream.pipeTo(new WritableStream({
			start(controller) {
				fsStream = fs.createWriteStream(cacheFile);
			},
			async write(chunk, controller) {
				await new Promise<void>((resolve, reject) => fsStream.write(chunk, err => err ? reject(err) : resolve()));
			},
			close() {
				fsStream.end();
				resolve(cacheFile);
			},
			abort() { }
		})).catch(reject);
	});

	await fs.promises.cp(cacheFile, saveName);
	await fs.promises.rm(cacheFile);
	return saveName;
}
